# sb::Plugin::Reorder - Plugin for Serene Bach
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

# 0.01 [2007/06/25] changed _reorder_list to load data correctly

package sb::Plugin::Reorder;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'admin',
		'name'    => 'Reorder list',
		'text'    => 'Reordering categories, links, recommendations and users',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => '0.01',
	},
	'file' => 'reorder.txt',
	'data' => undef,
);
sb::Plugin->register_admin_module(
	'mode'   => 'reorder',
	'level'  => 1,
	'module' => 'sb::Admin::Reorder',
);
package sb::Admin::Reorder;
# ==================================================
# // declaration for global variables
# ==================================================
use vars qw( @ISA );
# ==================================================
# // configuration for inheritance / dependancy
# ==================================================
use sb::Language ();
use sb::Interface ();
use sb::TemplateManager ();
use sb::Text ();
use sb::Data ();
use sb::Admin::List ();
@ISA = qw( sb::Admin::List );
# ==================================================
# // declaration for constant value
# ==================================================
sub TEMPLATE    (){ 'reorder.html' };
sub LIST_OPTION (){ '<option value="%s">%s</option>' };
sub JS_ARRAY    (){ 'gListOptions[%d] = new Array(%s);' };
# ==================================================
# // declaration for class member
# ==================================================
my @mOrderableList = ('Category','Link','Amazon','User');
# ==================================================
# // public functions - callback
# ==================================================
sub callback
{
	my $self = shift;
	return ( $self->{'regi'} ) 
		? $self->_reorder_list(@_)
		: $self->_open_options(@_);
}
# ==================================================
# // private functions - main routine
# ==================================================
sub _reorder_list
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi = sb::Interface->get;
	my $msg = '';
	if ($cgi->value('target_list') ne '')
	{
		my $list = $cgi->value('target_list');
		my @sels = split("\0",$cgi->value('change_order'));
		my %objs = sb::Data->load_as_hash($list,'cond'=>{'id'=>\@sels},'detail'=>'on');
		my @news = ();
		my $check = undef;
		for (my $i=0;$i<@sels;$i++)
		{
			my $obj = $objs{$sels[$i]};
			if ($obj)
			{
				$obj->order($#sels - $i);
				push(@news,$obj);
				$check = $obj->main if ($list eq 'Category' and $obj->main ne '');
			}
		}
		if ($list eq 'Category' and $check ne '')
		{
			my $main = sb::Data->load('Category','id'=>$check);
			if ($main) {
				$main->sub(join(',',map { $_->id } @news) . ',');
				sb::Data->update($main);
			}
		}
		sb::Data->update(@news) if (@news);
		$self->build_list('category_list') if ($list eq 'Category');
		$self->build_list('link_list') if ($list eq 'Link');
		$self->build_list('user_list') if ($list eq 'User');
		$msg = sb::Language->get->string('parts_editcomp');
	}
	else
	{
		$msg = sb::Language->get->string('error_reorderNonTarget');
	}
	return $self->_open_options('message'=>$msg);
}
sub _open_options
{
	my $self = shift;
	my %param = ( 'message' => '', @_ );
	my $cgi  = sb::Interface->get;
	my $lang = sb::Language->get;
	my $cms  = sb::TemplateManager->new(sb::Plugin->load_template('file'=>TEMPLATE));
	my $target = '';
	my $option = '';
	my $num = 1;
	my %cat = sb::Data->load_as_hash('Category');
	my %lnk = sb::Data->load_as_hash('Link');
	foreach my $list (@mOrderableList)
	{
		my $string = $lang->string('parts_reorder' . $list);
		my @elem = ();
		my @buf  = ();
		my @objs = sb::Data->load($list,'sort'=>'order','order'=>1);
		next if (!@objs);
		foreach my $obj (@objs)
		{
			if ($list eq 'Category' and $obj->main ne '')
			{
				push(@buf,$obj->main);
				next;
			}
			if ($list eq 'Link' and $obj->type =~ /^\d+$/)
			{
				push(@buf,$obj->type);
				next;
			}
			my $name = ($list eq 'User') ? $obj->real : $obj->name;
			$name =~ s/\'/\\\'/g;
			push(@elem,"'" . $obj->id . ':' . sb::Text->detitize($name) . "'");
		}
		$target .= sprintf(LIST_OPTION,$list,$string);
		$option .= sprintf(JS_ARRAY,$num,join(',',@elem)) . "\n";
		$num++;
		if (@buf)
		{
			my %cnt = ();
			@buf = grep(!$cnt{$_}++, @buf);
			foreach my $check (@buf)
			{
				@elem = ();
				my $list_name = ($list eq 'Category') 
					? $cat{$check}->fullname(\%cat) 
					: $lnk{$check}->name;
				foreach my $obj (@objs)
				{
					next if ($list eq 'Category' and $obj->main ne $check);
					next if ($list eq 'Link' and $obj->type ne $check);
					my $name = ($list eq 'Category') ? $obj->fullname(\%cat) : $obj->name;
					$name =~ s/\'/\\\'/g;
					push(@elem,"'" . $obj->id . ':' . sb::Text->detitize($name) . "'");
				} # end of foreach my $obj (@objs)
				if (@elem)
				{
					$target .= sprintf(LIST_OPTION,$list,$string . ' [' . $list_name . ']');
					$option .= sprintf(JS_ARRAY,$num,join(',',@elem)) . "\n";
					$num++;
				}
			} # end of foreach my $check (@buf)
		} # end of if (@buf)
	} # end of foreach my $list (@mOrderableList)
	$cms->num(0);
	$cms->tag('sb_target_list'=>$target);
	$cms->tag('sb_list_options'=>$option);
	if ($param{'message'} ne '')
	{
		$cms->num(0);
		$cms->tag('sb_process_message'=>$param{'message'});
		$cms->block('sb_process_message'=>1);
	}
	return sb::Interface->get->head('type'=>'text/html') . $self->set_main($cms->output);
}
1;
__END__
